"""Regression test for tangent space algorithm"""
from tests.scripts.nif import call_niftoaster
from tests.utils import BaseNifFileTestCase, assert_tuple_values

from pyffi.spells.nif.fix import SpellAddTangentSpace


class TestFixTangentSpaceNif(BaseNifFileTestCase):
    """Invoke the fix_texturepath spell check through nif toaster"""

    def setUp(self):
        super(TestFixTangentSpaceNif, self).setUp()
        self.src_name = "test_skincenterradius.nif"
        super(TestFixTangentSpaceNif, self).copyFile()
        super(TestFixTangentSpaceNif, self).readNifData()

    def test_explicit_fix_tangent_space(self):
        """run the spell that generates normals"""

        spell = SpellAddTangentSpace(data=self.data)
        spell.recurse()

        tangents = self.data.roots[0].children[1].data.tangents

        expected = [[-0.175, 0.967, 0.183],
                    [-0.367, 0.930, 0.000],
                    [-0.178, 0.918, 0.354],
                    [0.020, 0.994, 0.111],
                    [-0.992, 0.122, 0.035],
                    [-1.000, 0.028, 0.009],
                    [-0.363, -0.931, 0.039],
                    [-0.140, -0.990, 0.006],
                    [-0.239, -0.921, -0.306],
                    [-0.040, -0.997, -0.072],
                    [-0.992, 0.122, 0.035],
                    [-1.000, 0.028, 0.009],
                    [0.067, 0.997, 0.037],
                    [0.219, 0.975, 0.025],
                    [0.109, 0.993, -0.038],
                    [0.000, 0.961, 0.276],
                    [0.999, -0.034, 0.000],
                    [-0.492, 0.759, 0.428],
                    [-1.000, 0.015, 0.005],
                    [0.596, 0.274, 0.754],
                    [-0.690, 0.717, -0.094],
                    [0.915, -0.363, -0.177],
                    [0.640, -0.187, -0.745],
                    [-0.690, 0.717, -0.094],
                    [-0.960, 0.155, -0.231],
                    [-0.714, 0.653, 0.254],
                    [-0.714, 0.653, 0.254],
                    [0.141, 0.988, -0.069],
                    [-0.492, 0.759, 0.428],
                    [0.915, -0.363, -0.177],
                    [-0.690, 0.717, -0.094],
                    [0.069, 0.997, 0.030],
                    [0.014, 0.996, 0.093],
                    [-0.235, 0.972, 0.029],
                    [0.283, 0.949, -0.140],
                    [0.517, 0.820, -0.247],
                    [0.944, -0.234, 0.234],
                    [-0.271, 0.959, 0.085],
                    [0.498, -0.370, 0.784],
                    [0.476, -0.404, 0.781],
                    [0.944, -0.234, 0.234],
                    [0.517, 0.820, -0.247],
                    [-0.791, 0.230, 0.568],
                    [-0.714, 0.653, 0.254],
                    [0.915, -0.363, -0.177],
                    [0.136, 0.987, 0.083],
                    [0.336, 0.941, 0.049],
                    [1.000, -0.031, 0.005],
                    [0.187, 0.982, 0.013],
                    [0.160, 0.983, -0.089],
                    [1.000, -0.031, 0.005],
                    [1.000, -0.031, 0.005],
                    [0.233, -0.788, -0.570],
                    [0.007, -1.000, 0.004],
                    [0.048, -0.997, 0.068],
                    [-0.064, -0.965, -0.256],
                    [0.999, -0.034, 0.000],
                    [-0.572, -0.732, -0.370],
                    [-1.000, 0.015, 0.005],
                    [-0.643, -0.691, -0.331],
                    [0.640, -0.187, -0.745],
                    [0.915, -0.363, -0.177],
                    [-0.978, -0.146, -0.147],
                    [-0.643, -0.691, -0.331],
                    [-0.727, -0.644, -0.240],
                    [-0.727, -0.644, -0.240],
                    [0.081, -0.991, 0.109],
                    [0.915, -0.363, -0.177],
                    [-0.572, -0.732, -0.370],
                    [-0.040, -0.998, -0.044],
                    [0.020, -1.000, 0.023],
                    [-0.233, -0.973, 0.010],
                    [0.283, 0.949, -0.140],
                    [0.460, -0.842, 0.282],
                    [0.460, -0.842, 0.282],
                    [-0.277, -0.960, -0.038],
                    [0.460, -0.842, 0.282],
                    [0.915, -0.363, -0.177],
                    [-0.212, -0.656, -0.724],
                    [0.315, -0.885, -0.344],
                    [1.000, -0.031, 0.005],
                    [0.175, -0.984, 0.040],
                    [0.102, -0.986, 0.132],
                    [1.000, -0.031, 0.005],
                    [1.000, -0.031, 0.005],
                    [-0.084, 0.992, -0.092],
                    [-0.072, 0.988, -0.136],
                    [-0.156, 0.975, -0.157],
                    [-0.367, 0.930, 0.000],
                    [-0.029, 1.000, 0.004],
                    [-1.000, 0.016, -0.001],
                    [-1.000, 0.028, 0.000],
                    [-0.098, -0.988, 0.122],
                    [-0.098, -0.992, 0.081],
                    [-0.167, -0.975, 0.146],
                    [-0.363, -0.931, 0.039],
                    [-0.140, -0.990, 0.006],
                    [-1.000, 0.028, 0.000],
                    [-1.000, 0.016, -0.001],
                    [0.977, 0.122, -0.173],
                    [0.151, -0.978, -0.142],
                    [0.995, 0.055, 0.077],
                    [0.353, 0.931, 0.087],
                    [0.995, 0.055, 0.077],
                    [0.151, -0.978, -0.142],
                    [1.000, -0.016, -0.003],
                    [0.106, 0.980, 0.167],
                    [0.387, -0.889, -0.246],
                    [1.000, -0.016, -0.003],
                    [0.084, -0.980, -0.181],
                    [-0.270, -0.931, -0.246],
                    [0.995, 0.055, 0.077],
                    [0.977, 0.122, -0.173],
                    [0.239, -0.737, 0.632],
                    [0.379, 0.646, 0.663],
                    [-0.270, -0.931, -0.246],
                    [0.379, 0.646, 0.663],
                    [-0.270, -0.931, -0.246],
                    [-0.831, 0.538, 0.143],
                    [0.995, 0.055, 0.077],
                    [0.379, 0.646, 0.663],
                    [0.977, 0.122, -0.173],
                    [0.152, 0.986, 0.072],
                    [-0.245, 0.945, -0.217],
                    [-0.060, 0.998, -0.022],
                    [-0.918, -0.246, 0.311],
                    [-0.270, -0.931, -0.246],
                    [0.379, 0.646, 0.663],
                    [0.995, 0.055, 0.077],
                    [-0.186, 0.972, -0.145],
                    [-0.707, -0.506, 0.494],
                    [0.219, 0.975, 0.025],
                    [0.336, 0.941, 0.049],
                    [0.635, 0.768, -0.082],
                    [0.965, 0.169, -0.199],
                    [0.965, 0.169, -0.199],
                    [-0.270, -0.931, -0.246],
                    [0.995, 0.055, 0.077],
                    [0.965, 0.169, -0.199],
                    [0.239, -0.737, 0.632],
                    [-0.270, -0.931, -0.246],
                    [0.635, 0.768, -0.082],
                    [-0.918, -0.246, 0.311],
                    [0.392, -0.810, 0.435],
                    [0.654, -0.689, 0.311],
                    [-0.707, -0.506, 0.494],
                    [0.635, 0.768, -0.082],
                    [0.995, 0.055, 0.077],
                    [0.353, 0.931, 0.087],
                    [-0.899, -0.283, 0.336],
                    [-0.747, 0.626, -0.222],
                    [0.654, -0.689, 0.311],
                    [0.941, -0.337, 0.019],
                    [-0.270, -0.931, -0.246],
                    [-0.747, 0.558, 0.361],
                    [-0.899, -0.283, 0.336],
                    [-0.690, 0.717, -0.094],
                    [0.596, 0.274, 0.754],
                    [-0.747, 0.626, -0.222],
                    [-0.747, 0.626, -0.222],
                    [-0.690, 0.717, -0.094],
                    [-0.747, 0.626, -0.222],
                    [-0.872, 0.470, -0.139],
                    [0.136, 0.987, 0.083],
                    [-0.235, 0.972, 0.029],
                    [-0.003, 1.000, -0.006],
                    [-0.271, 0.959, 0.085],
                    [-0.872, 0.470, -0.139],
                    [-0.960, 0.155, -0.231],
                    [0.995, 0.055, 0.077],
                    [0.187, 0.982, 0.013],
                    [0.872, 0.449, 0.195],
                    [0.134, 0.904, 0.405],
                    [1.000, -0.005, -0.002],
                    [-0.661, -0.498, -0.561],
                    [-0.189, -0.408, -0.893],
                    [-0.461, -0.451, -0.764],
                    [0.399, 0.907, 0.132],
                    [0.399, 0.907, 0.132],
                    [-0.990, -0.112, -0.081],
                    [-0.641, -0.460, -0.614],
                    [-0.976, -0.092, -0.199],
                    [-0.990, -0.112, -0.081],
                    [-0.661, -0.498, -0.561],
                    [0.233, -0.788, -0.570],
                    [0.315, -0.885, -0.344],
                    [-0.108, -0.598, -0.794],
                    [0.977, 0.122, -0.173],
                    [0.399, 0.907, 0.132],
                    [-0.270, -0.931, -0.246],
                    [0.399, 0.907, 0.132],
                    [-0.824, -0.426, -0.374],
                    [-0.976, -0.092, -0.199],
                    [0.892, -0.324, -0.314],
                    [0.399, 0.907, 0.132],
                    [-0.824, -0.426, -0.374],
                    [0.892, -0.324, -0.314],
                    [-0.831, 0.538, 0.143],
                    [-0.824, -0.426, -0.374],
                    [0.399, 0.907, 0.132],
                    [-0.824, -0.426, -0.374],
                    [-0.108, -0.598, -0.794],
                    [0.892, -0.324, -0.314],
                    [-0.831, 0.538, 0.143],
                    [1.000, -0.005, -0.002],
                    [0.213, -0.816, -0.537],
                    [-0.791, 0.230, 0.568],
                    [0.392, -0.810, 0.435],
                    [-0.839, -0.529, 0.129],
                    [-0.990, -0.112, -0.081],
                    [0.654, -0.689, 0.311],
                    [0.654, -0.689, 0.311],
                    [0.892, -0.324, -0.314],
                    [-0.108, -0.598, -0.794],
                    [0.387, -0.889, -0.246],
                    [0.941, -0.337, 0.019],
                    [0.977, 0.122, -0.173],
                    [-0.976, -0.092, -0.199],
                    [-0.839, -0.529, 0.129],
                    [0.941, -0.337, 0.019],
                    [0.387, -0.889, -0.246],
                    [-0.108, -0.598, -0.794],
                    [0.315, -0.885, -0.344],
                    [0.387, -0.889, -0.246],
                    [-0.899, -0.283, 0.336],
                    [-0.747, 0.558, 0.361],
                    [-0.823, -0.290, -0.489],
                    [-0.594, -0.015, -0.804],
                    [-0.643, -0.691, -0.331],
                    [0.640, -0.187, -0.745],
                    [-0.839, -0.529, 0.129],
                    [-0.823, -0.290, -0.489],
                    [-0.643, -0.691, -0.331],
                    [-0.839, -0.529, 0.129],
                    [-0.971, -0.196, -0.137],
                    [-0.212, -0.656, -0.724],
                    [-0.899, -0.283, 0.336],
                    [-0.233, -0.973, 0.010],
                    [-0.028, -1.000, -0.008],
                    [-0.277, -0.960, -0.038],
                    [0.392, -0.810, 0.435],
                    [-0.990, -0.112, -0.081],
                    [0.654, -0.689, 0.311],
                    [-0.971, -0.196, -0.137],
                    [-0.643, -0.691, -0.331],
                    [-0.978, -0.146, -0.147],
                    [-0.971, -0.196, -0.137],
                    [0.315, -0.885, -0.344],
                    [0.175, -0.984, 0.040],
                    [0.872, 0.449, 0.195],
                    [0.213, -0.816, -0.537],
                    [1.000, -0.005, -0.002],
                    [0.872, 0.449, 0.195],
                    [0.213, -0.816, -0.537],
                    [-0.226, -0.973, -0.035],
                    [-0.296, -0.950, -0.100],
                    [-0.999, 0.028, -0.021],
                    [-0.997, 0.066, 0.026],
                    [-0.999, 0.028, -0.021],
                    [-0.997, 0.066, 0.026],
                    [-0.087, 0.994, 0.065],
                    [-0.123, 0.989, 0.087],
                    [0.058, -0.998, 0.018],
                    [-0.159, -0.986, -0.046],
                    [-0.356, -0.908, -0.223],
                    [-0.356, -0.908, -0.223],
                    [-0.067, 0.994, 0.084],
                    [0.040, 0.999, 0.014],
                    [-0.087, 0.994, 0.065],
                    [-0.123, 0.989, 0.087],
                    [-0.019, 1.000, 0.014],
                    [-0.033, 0.999, 0.005],
                    [-0.998, 0.017, 0.058],
                    [-1.000, 0.009, 0.013],
                    [-0.998, 0.017, 0.058],
                    [-1.000, 0.009, 0.013],
                    [-0.051, -0.998, 0.048],
                    [-0.095, -0.992, 0.078],
                    [-0.226, -0.973, -0.035],
                    [-0.296, -0.950, -0.100],
                    [0.040, 0.999, 0.014],
                    [0.014, 0.999, 0.037],
                    [0.090, 0.994, 0.062],
                    [0.058, -0.998, 0.018],
                    [0.025, -0.999, -0.029],
                    [0.090, 0.994, 0.062],
                    [-0.271, 0.953, 0.137],
                    [-0.200, 0.976, 0.086],
                    [-0.999, -0.033, 0.018],
                    [-0.997, -0.075, -0.030],
                    [-0.997, -0.075, -0.030],
                    [-0.999, -0.033, 0.018],
                    [-0.103, -0.993, -0.052],
                    [-0.146, -0.987, -0.068],
                    [-0.200, 0.976, 0.086],
                    [0.039, 0.998, 0.055],
                    [-0.118, 0.987, 0.108],
                    [-0.118, 0.987, 0.108],
                    [-0.314, 0.920, 0.233],
                    [-0.314, 0.920, 0.233],
                    [-0.045, -0.993, -0.107],
                    [0.042, -0.999, 0.012],
                    [-0.146, -0.987, -0.068],
                    [-0.103, -0.993, -0.052],
                    [-0.007, -0.999, 0.054],
                    [-0.004, -0.999, 0.050],
                    [-0.990, -0.137, -0.019],
                    [-0.110, 0.993, -0.041],
                    [-0.990, -0.137, -0.019],
                    [-0.072, 0.997, -0.011],
                    [-0.271, 0.953, 0.137],
                    [0.042, -0.999, 0.012],
                    [0.046, -0.999, 0.022],
                    [0.075, -0.997, -0.007],
                    [-0.013, 1.000, 0.022],
                    [0.075, -0.997, -0.007],
                    [0.077, 0.997, 0.005],
                    [0.078, 0.994, 0.081],
                    [0.996, -0.077, 0.054],
                    [0.996, -0.077, 0.054],
                    [0.015, -1.000, -0.005],
                    [-0.008, -0.999, 0.040],
                    [-0.008, -0.999, 0.040],
                    [0.949, 0.315, 0.004],
                    [0.949, 0.315, 0.004],
                    [0.077, 0.997, 0.005],
                    [0.207, 0.978, -0.036],
                    [0.069, 0.994, 0.084],
                    [0.998, -0.050, 0.044],
                    [0.998, -0.050, 0.044],
                    [0.079, -0.994, -0.074],
                    [0.177, -0.984, 0.012],
                    [0.177, -0.984, 0.012],
                    [0.106, -0.976, 0.188],
                    [0.992, -0.044, 0.119],
                    [0.992, -0.044, 0.119],
                    [0.207, 0.978, -0.036],
                    [0.038, 0.999, 0.015],
                    [0.703, 0.711, -0.015],
                    [-0.011, 1.000, 0.016],
                    [-0.914, -0.144, 0.380],
                    [-0.124, -0.991, 0.049],
                    [-0.124, -0.991, 0.049],
                    [-0.914, -0.144, 0.380],
                    [-0.062, -0.995, 0.082],
                    [-0.100, -0.987, 0.129],
                    [-0.068, -0.997, 0.036],
                    [-0.058, -0.998, 0.031],
                    [-0.068, -0.997, 0.036],
                    [-0.058, -0.998, 0.031],
                    [-0.027, -0.999, 0.031],
                    [-0.023, -0.999, 0.041],
                    [0.118, -0.990, 0.080],
                    [0.176, 0.982, -0.064],
                    [0.118, -0.990, 0.080],
                    [0.176, 0.982, -0.064],
                    [0.049, 0.998, -0.025],
                    [0.017, 1.000, -0.007],
                    [0.038, 0.999, 0.015],
                    [0.703, 0.711, -0.015],
                    [0.049, 0.999, 0.015],
                    [0.025, 0.999, 0.047],
                    [0.998, -0.001, -0.058],
                    [0.998, -0.001, -0.058],
                    [0.054, -0.997, -0.063],
                    [0.112, -0.993, 0.042],
                    [0.112, -0.993, 0.042],
                    [0.025, -0.993, 0.115],
                    [0.436, 0.899, 0.049],
                    [0.436, 0.899, 0.049],
                    [0.049, 0.999, 0.015],
                    [0.069, -0.997, 0.019],
                    [0.113, -0.993, -0.030],
                    [0.995, 0.087, -0.054],
                    [0.995, 0.087, -0.054],
                    [-0.008, 0.999, 0.036],
                    [0.004, 1.000, -0.002],
                    [0.004, 1.000, -0.002],
                    [0.997, -0.082, -0.012],
                    [0.997, -0.082, -0.012],
                    [0.069, -0.997, 0.019],
                    [0.200, -0.978, 0.061],
                    [0.106, -0.994, -0.039],
                    [0.997, 0.062, -0.043],
                    [0.997, 0.062, -0.043],
                    [0.058, 0.994, 0.096],
                    [0.172, 0.985, 0.029],
                    [0.119, 0.977, -0.175],
                    [0.172, 0.985, 0.029],
                    [0.991, 0.053, -0.122],
                    [0.991, 0.053, -0.122],
                    [0.200, -0.978, 0.061],
                    [0.703, 0.711, -0.015],
                    [0.177, -0.984, 0.005],
                    [0.060, -0.998, -0.005],
                    [-0.005, -0.998, 0.059],
                    [-0.980, -0.089, -0.179],
                    [0.617, -0.764, 0.189],
                    [0.617, -0.764, 0.189],
                    [-0.980, -0.089, -0.179],
                    [0.019, 0.990, -0.137],
                    [0.140, 0.945, -0.295],
                    [0.617, -0.764, 0.189],
                    [0.019, 0.990, -0.137],
                    [0.140, 0.945, -0.295],
                    [-0.063, 0.998, 0.002],
                    [-0.044, 0.999, -0.004],
                    [-0.044, 0.999, -0.004],
                    [-0.063, 0.998, 0.002],
                    [-0.016, 1.000, -0.026],
                    [-0.011, 0.999, -0.036],
                    [0.199, 0.977, -0.080],
                    [0.227, -0.973, 0.042],
                    [0.227, -0.973, 0.042],
                    [0.199, 0.977, -0.080],
                    [0.063, -0.998, -0.000],
                    [0.133, -0.990, 0.044],
                    [0.177, -0.984, 0.005],
                    [0.703, 0.711, -0.015],
                    [0.048, -0.999, 0.010],
                    [0.063, -0.998, -0.006],
                    [0.998, 0.023, 0.060],
                    [0.998, 0.023, 0.060],
                    [0.035, 0.996, 0.079],
                    [0.113, 0.994, -0.003],
                    [0.113, 0.994, -0.003],
                    [0.037, 0.994, -0.107],
                    [0.521, -0.849, -0.092],
                    [0.521, -0.849, -0.092],
                    [0.048, -0.999, 0.010],
                    [-0.584, 0.689, 0.429],
                    [-0.273, 0.957, -0.098],
                    [-0.424, 0.166, 0.890],
                    [0.315, 0.750, 0.581],
                    [0.494, 0.393, 0.775],
                    [-0.584, 0.689, 0.429],
                    [0.315, 0.750, 0.581],
                    [-0.219, -0.852, 0.476],
                    [0.660, 0.348, 0.666],
                    [0.660, 0.348, 0.666],
                    [-0.244, -0.938, 0.245],
                    [-0.219, -0.852, 0.476],
                    [-0.266, -0.960, -0.084],
                    [-0.109, -0.988, -0.105],
                    [0.029, -0.996, -0.078],
                    [-0.273, 0.957, -0.098],
                    [0.121, 0.989, 0.087],
                    [0.081, 0.992, -0.097],
                    [0.067, 0.992, -0.107],
                    [1.000, -0.019, -0.014],
                    [0.131, -0.983, 0.126],
                    [0.131, -0.983, 0.126],
                    [1.000, -0.019, -0.014],
                    [0.091, -0.995, 0.045],
                    [0.082, -0.991, 0.109],
                    [-0.244, -0.938, 0.245],
                    [0.130, 0.990, 0.051],
                    [0.940, -0.325, 0.105],
                    [0.940, -0.325, 0.105],
                    [0.146, -0.988, -0.053],
                    [0.006, 0.990, 0.140],
                    [-0.157, 0.988, 0.005],
                    [-0.157, 0.988, 0.005],
                    [0.029, -0.996, -0.078],
                    [0.146, -0.988, -0.053],
                    [0.029, -0.996, -0.078],
                    [-0.109, -0.988, -0.105],
                    [-0.266, -0.960, -0.084],
                    [-0.356, -0.933, 0.042],
                    [-0.244, -0.938, 0.245],
                    [0.091, -0.995, 0.045],
                    [0.121, -0.983, -0.139],
                    [-0.219, -0.852, 0.476],
                    [-0.244, -0.938, 0.245],
                    [0.121, -0.983, -0.139],
                    [-0.219, -0.852, 0.476],
                    [0.815, 0.561, 0.149],
                    [0.494, 0.393, 0.775],
                    [0.494, 0.393, 0.775],
                    [0.815, 0.561, 0.149],
                    [0.003, 0.943, 0.332],
                    [0.078, 0.954, 0.288],
                    [-0.584, 0.689, 0.429],
                    [0.121, 0.989, 0.087],
                    [-0.273, 0.957, -0.098],
                    [-0.994, -0.079, 0.081],
                    [-0.994, -0.079, 0.081],
                    [0.130, 0.990, 0.051],
                    [-0.356, -0.933, 0.042],
                    [0.006, 0.990, 0.140],
                    [-0.227, -0.973, -0.034],
                    [-0.441, -0.837, 0.325],
                    [-0.424, -0.166, -0.890],
                    [0.428, -0.712, -0.556],
                    [-0.441, -0.837, 0.325],
                    [0.442, 0.522, 0.730],
                    [0.428, -0.712, -0.556],
                    [0.442, 0.522, 0.730],
                    [-0.555, 0.831, 0.040],
                    [0.701, -0.303, -0.646],
                    [0.442, 0.522, 0.730],
                    [0.701, -0.303, -0.646],
                    [0.428, -0.712, -0.556],
                    [-0.189, 0.929, 0.318],
                    [0.701, -0.303, -0.646],
                    [-0.555, 0.831, 0.040],
                    [-0.109, 0.989, 0.104],
                    [-0.228, 0.973, 0.027],
                    [0.026, 0.997, 0.072],
                    [0.121, -0.979, -0.166],
                    [-0.227, -0.973, -0.034],
                    [0.231, -0.965, -0.127],
                    [0.255, -0.962, -0.092],
                    [0.992, 0.122, -0.000],
                    [0.327, 0.941, -0.088],
                    [0.992, 0.122, -0.000],
                    [0.327, 0.941, -0.088],
                    [0.126, 0.992, 0.019],
                    [0.149, 0.989, -0.018],
                    [0.149, 0.989, -0.018],
                    [-0.189, 0.929, 0.318],
                    [0.126, 0.992, 0.019],
                    [0.144, -0.964, -0.224],
                    [0.981, 0.165, -0.097],
                    [0.981, 0.165, -0.097],
                    [0.149, 0.984, 0.096],
                    [-0.022, -0.974, -0.225],
                    [0.257, -0.952, -0.167],
                    [0.257, -0.952, -0.167],
                    [0.026, 0.997, 0.072],
                    [0.149, 0.984, 0.096],
                    [-0.321, 0.946, -0.042],
                    [0.056, 0.972, 0.230],
                    [-0.555, 0.831, 0.040],
                    [0.725, -0.665, -0.181],
                    [0.442, 0.522, 0.730],
                    [0.725, -0.665, -0.181],
                    [-0.441, -0.837, 0.325],
                    [0.108, -0.960, -0.257],
                    [0.442, 0.522, 0.730],
                    [0.121, -0.979, -0.166],
                    [-0.441, -0.837, 0.325],
                    [-0.992, 0.094, -0.078],
                    [-0.992, 0.094, -0.078],
                    [0.144, -0.964, -0.224],
                    [-0.321, 0.946, -0.042],
                    [-0.022, -0.974, -0.225],
                    [0.268, -0.487, -0.831],
                    [-0.618, 0.310, -0.723],
                    [0.399, -0.465, -0.791],
                    [0.097, 0.661, -0.744],
                    [0.322, 0.669, -0.670],
                    [0.268, -0.487, -0.831],
                    [0.262, 0.381, -0.887],
                    [0.322, 0.669, -0.670],
                    [0.097, 0.661, -0.744],
                    [0.399, -0.465, -0.791],
                    [-0.121, 0.583, -0.804],
                    [-0.175, 0.680, -0.712],
                    [0.004, 0.655, -0.756],
                    [-0.006, 0.596, -0.803],
                    [0.004, 0.655, -0.756],
                    [-0.711, 0.450, -0.540],
                    [-0.006, 0.596, -0.803],
                    [-0.711, 0.450, -0.540],
                    [0.375, -0.927, -0.013],
                    [0.584, 0.174, -0.793],
                    [-0.006, 0.596, -0.803],
                    [0.375, -0.927, -0.013],
                    [0.375, -0.927, -0.013],
                    [0.399, -0.465, -0.791],
                    [-0.618, 0.310, -0.723],
                    [-0.175, 0.680, -0.712],
                    [-0.618, 0.310, -0.723],
                    [0.322, 0.669, -0.670],
                    [0.004, 0.655, -0.756],
                    [0.322, 0.669, -0.670],
                    [0.262, 0.381, -0.887],
                    [0.399, -0.465, -0.791],
                    [-0.121, 0.583, -0.804],
                    [0.004, 0.655, -0.756],
                    [-0.006, 0.596, -0.803],
                    [-0.190, -0.442, 0.876],
                    [0.269, 0.488, 0.831],
                    [0.340, 0.311, 0.888],
                    [0.300, -0.533, 0.791],
                    [0.097, -0.661, 0.744],
                    [0.149, -0.378, 0.914],
                    [0.269, 0.488, 0.831],
                    [0.300, -0.533, 0.791],
                    [0.097, -0.661, 0.744],
                    [0.340, 0.311, 0.888],
                    [0.214, -0.559, 0.801],
                    [-0.121, -0.581, 0.805],
                    [-0.055, -0.560, 0.826],
                    [-0.055, -0.560, 0.826],
                    [0.549, -0.571, 0.610],
                    [0.104, -0.574, 0.812],
                    [0.104, -0.574, 0.812],
                    [0.549, -0.571, 0.610],
                    [0.308, 0.909, -0.281],
                    [0.827, -0.017, 0.562],
                    [0.340, 0.311, 0.888],
                    [0.308, 0.909, -0.281],
                    [-0.190, -0.442, 0.876],
                    [0.214, -0.559, 0.801],
                    [-0.190, -0.442, 0.876],
                    [0.300, -0.533, 0.791],
                    [0.300, -0.533, 0.791],
                    [-0.055, -0.560, 0.826],
                    [0.149, -0.378, 0.914],
                    [-0.055, -0.560, 0.826],
                    [0.104, -0.574, 0.812],
                    [0.104, -0.574, 0.812],
                    [0.308, 0.909, -0.281],
                    [0.340, 0.311, 0.888],
                    [-0.121, -0.581, 0.805],
                    [0.962, -0.232, 0.145],
                    [0.999, 0.051, 0.002],
                    [0.971, -0.195, 0.136],
                    [0.971, -0.195, 0.136],
                    [0.999, 0.051, 0.002],
                    [0.997, -0.068, -0.030],
                    [0.996, 0.059, -0.062],
                    [-0.970, 0.086, 0.225],
                    [0.996, 0.059, -0.062],
                    [-0.970, 0.086, 0.225],
                    [-0.974, 0.103, 0.204],
                    [-0.980, 0.157, 0.118],
                    [-0.974, 0.103, 0.204],
                    [-0.980, 0.157, 0.118],
                    [-0.993, -0.018, -0.121],
                    [0.962, -0.232, 0.145],
                    [-0.993, -0.018, -0.121],
                    [0.962, -0.232, 0.145],
                    [0.971, -0.195, 0.136],
                    [-0.999, -0.038, 0.033],
                    [-0.990, 0.137, -0.041],
                    [-0.990, 0.137, -0.041],
                    [-0.993, -0.018, -0.121],
                    [-0.999, -0.038, 0.033],
                    [0.971, -0.195, 0.136],
                    [-0.956, -0.127, 0.264],
                    [-0.994, 0.090, 0.057],
                    [-0.987, -0.018, 0.163],
                    [-0.994, 0.090, 0.057],
                    [0.165, 0.808, -0.566],
                    [-0.987, -0.018, 0.163],
                    [0.165, 0.808, -0.566],
                    [0.995, 0.101, 0.004],
                    [1.000, -0.018, -0.023],
                    [0.997, 0.078, -0.026],
                    [0.939, 0.200, 0.279],
                    [-0.990, 0.051, 0.132],
                    [0.939, 0.200, 0.279],
                    [-0.990, 0.051, 0.132],
                    [-0.990, -0.131, -0.061],
                    [-0.966, 0.179, 0.187],
                    [-0.990, -0.131, -0.061],
                    [-0.966, 0.179, 0.187],
                    [-0.991, -0.007, 0.134],
                    [-0.977, 0.188, -0.101],
                    [-0.994, 0.090, 0.057],
                    [-0.977, 0.188, -0.101],
                    [0.995, 0.101, 0.004],
                    [-0.987, -0.018, 0.163],
                    [0.994, -0.101, -0.033],
                    [0.919, 0.249, 0.306],
                    [0.919, 0.249, 0.306],
                    [0.939, 0.200, 0.279],
                    [-0.897, 0.251, -0.365],
                    [-0.897, 0.251, -0.365],
                    [-0.897, 0.251, -0.365],
                    [-0.956, -0.127, 0.264],
                    [-0.753, -0.592, -0.286],
                    [-0.955, 0.215, -0.203],
                    [0.863, -0.429, 0.266],
                    [0.966, -0.227, 0.120],
                    [-0.753, -0.592, -0.286],
                    [0.863, -0.429, 0.266],
                    [0.948, -0.107, 0.299],
                    [-0.040, -0.946, -0.322],
                    [0.980, -0.179, 0.084],
                    [0.948, -0.107, 0.299],
                    [0.980, -0.179, 0.084],
                    [1.000, -0.004, 0.002],
                    [0.224, -0.788, -0.573],
                    [-0.964, -0.125, -0.234],
                    [-0.964, -0.125, -0.234],
                    [0.224, -0.788, -0.573],
                    [-0.971, -0.138, -0.197],
                    [-0.981, -0.158, -0.110],
                    [-0.981, -0.158, -0.110],
                    [-0.971, -0.138, -0.197],
                    [-0.066, -0.639, -0.766],
                    [-0.040, -0.946, -0.322],
                    [-0.040, -0.946, -0.322],
                    [-0.066, -0.639, -0.766],
                    [0.980, -0.179, 0.084],
                    [0.993, 0.089, -0.083],
                    [-0.126, -0.741, -0.659],
                    [-0.126, -0.741, -0.659],
                    [0.993, 0.089, -0.083],
                    [0.966, -0.227, 0.120],
                    [1.000, 0.019, 0.003],
                    [0.986, 0.164, -0.002],
                    [0.986, 0.164, -0.002],
                    [-0.952, 0.304, -0.019],
                    [-0.952, 0.304, -0.019],
                    [-0.998, 0.011, -0.058],
                    [-0.753, -0.592, -0.286],
                    [-0.126, -0.741, -0.659],
                    [0.966, -0.227, 0.120],
                    [0.863, -0.429, 0.266],
                    [0.994, 0.105, 0.010],
                    [0.921, -0.244, -0.304],
                    [0.986, 0.164, -0.002],
                    [0.921, -0.244, -0.304],
                    [-0.936, 0.340, 0.089],
                    [-0.952, 0.304, -0.019],
                    [0.986, 0.164, -0.002],
                    [-0.936, 0.340, 0.089],
                    [-0.936, 0.340, 0.089],
                    [-0.955, 0.215, -0.203]]

        for t, e in zip(tangents, expected):
            assert_tuple_values(t.as_tuple(), e)

        bitangents = self.data.roots[0].children[1].data.bitangents

        expected = [[0.541, -0.061, 0.839],
                    [-0.097, -0.038, 0.995],
                    [0.796, -0.077, 0.600],
                    [0.832, -0.078, 0.549],
                    [0.126, 0.954, 0.271],
                    [0.030, 0.952, 0.304],
                    [0.111, -0.001, 0.994],
                    [0.076, -0.005, 0.997],
                    [-0.780, -0.006, 0.626],
                    [-0.831, -0.007, 0.556],
                    [0.126, 0.954, 0.271],
                    [0.030, 0.952, 0.304],
                    [0.777, -0.075, 0.625],
                    [0.102, -0.048, 0.994],
                    [0.890, -0.081, 0.448],
                    [0.954, -0.083, 0.289],
                    [-0.034, -0.999, 0.011],
                    [0.812, 0.221, 0.541],
                    [0.016, 0.948, 0.317],
                    [0.307, -0.946, 0.102],
                    [-0.063, 0.070, 0.996],
                    [-0.404, -0.822, -0.401],
                    [-0.241, -0.970, 0.037],
                    [-0.063, 0.070, 0.996],
                    [-0.234, 0.001, 0.972],
                    [0.643, 0.468, 0.606],
                    [0.643, 0.468, 0.606],
                    [0.822, -0.078, 0.565],
                    [0.812, 0.221, 0.541],
                    [-0.404, -0.822, -0.401],
                    [-0.063, 0.070, 0.996],
                    [0.660, -0.068, 0.748],
                    [0.739, -0.073, 0.670],
                    [-0.083, -0.050, 0.995],
                    [0.959, -0.280, 0.041],
                    [0.788, -0.343, 0.511],
                    [-0.331, -0.663, 0.672],
                    [0.060, -0.072, 0.996],
                    [-0.867, -0.245, 0.434],
                    [-0.879, -0.182, 0.441],
                    [-0.331, -0.663, 0.672],
                    [0.788, -0.343, 0.511],
                    [-0.611, -0.238, -0.755],
                    [0.643, 0.468, 0.606],
                    [-0.404, -0.822, -0.401],
                    [-0.394, -0.023, 0.919],
                    [-0.017, -0.046, 0.999],
                    [-0.031, -0.989, 0.145],
                    [0.202, -0.051, 0.978],
                    [0.800, -0.076, 0.596],
                    [-0.031, -0.986, 0.163],
                    [-0.031, -0.985, 0.170],
                    [0.033, -0.579, 0.814],
                    [-0.776, -0.003, 0.631],
                    [-0.895, -0.012, 0.446],
                    [-0.951, -0.019, 0.309],
                    [-0.034, -0.999, 0.011],
                    [-0.757, 0.297, 0.583],
                    [0.016, 0.948, 0.317],
                    [-0.246, -0.222, 0.943],
                    [-0.241, -0.970, 0.037],
                    [-0.404, -0.822, -0.401],
                    [-0.140, -0.057, 0.988],
                    [-0.246, -0.222, 0.943],
                    [-0.627, 0.479, 0.614],
                    [-0.627, 0.479, 0.614],
                    [-0.830, -0.007, 0.558],
                    [-0.404, -0.822, -0.401],
                    [-0.757, 0.297, 0.583],
                    [-0.738, 0.000, 0.675],
                    [-0.663, 0.004, 0.748],
                    [0.093, -0.012, 0.996],
                    [0.959, -0.280, 0.041],
                    [-0.822, -0.284, 0.493],
                    [-0.822, -0.284, 0.493],
                    [-0.046, -0.026, 0.999],
                    [-0.822, -0.284, 0.493],
                    [-0.404, -0.822, -0.401],
                    [0.365, -0.741, 0.564],
                    [0.118, -0.323, 0.939],
                    [-0.031, -0.989, 0.145],
                    [-0.212, 0.002, 0.977],
                    [-0.809, -0.005, 0.588],
                    [-0.031, -0.986, 0.163],
                    [-0.031, -0.985, 0.170],
                    [-0.775, -0.007, 0.632],
                    [-0.882, 0.001, 0.471],
                    [-0.735, -0.008, 0.678],
                    [-0.097, -0.038, 0.995],
                    [-0.510, -0.018, 0.860],
                    [-0.016, -0.996, 0.092],
                    [-0.028, -0.999, -0.017],
                    [0.880, -0.029, 0.475],
                    [0.773, -0.025, 0.633],
                    [0.732, -0.023, 0.681],
                    [0.111, -0.001, 0.994],
                    [0.076, -0.005, 0.997],
                    [-0.028, -0.999, -0.017],
                    [-0.016, -0.996, 0.092],
                    [-0.046, 0.920, 0.390],
                    [0.989, 0.149, 0.022],
                    [-0.081, 0.918, 0.388],
                    [-0.681, 0.192, 0.707],
                    [-0.081, 0.918, 0.388],
                    [0.989, 0.149, 0.022],
                    [0.016, 0.988, 0.157],
                    [-0.840, -0.002, 0.543],
                    [0.717, 0.122, 0.687],
                    [0.016, 0.988, 0.157],
                    [0.842, -0.028, 0.538],
                    [0.691, -0.365, 0.624],
                    [-0.081, 0.918, 0.388],
                    [-0.046, 0.920, 0.390],
                    [0.905, 0.404, 0.129],
                    [-0.924, 0.300, 0.236],
                    [0.691, -0.365, 0.624],
                    [-0.924, 0.300, 0.236],
                    [0.691, -0.365, 0.624],
                    [-0.521, -0.842, 0.142],
                    [-0.081, 0.918, 0.388],
                    [-0.924, 0.300, 0.236],
                    [-0.046, 0.920, 0.390],
                    [-0.286, -0.026, 0.958],
                    [-0.669, -0.003, 0.743],
                    [-0.559, -0.016, 0.829],
                    [-0.363, 0.836, -0.411],
                    [0.691, -0.365, 0.624],
                    [-0.924, 0.300, 0.236],
                    [-0.081, 0.918, 0.388],
                    [-0.631, -0.005, 0.776],
                    [-0.695, 0.628, -0.350],
                    [0.102, -0.048, 0.994],
                    [-0.017, -0.046, 0.999],
                    [-0.356, 0.385, 0.852],
                    [-0.120, 0.964, 0.237],
                    [-0.120, 0.964, 0.237],
                    [0.691, -0.365, 0.624],
                    [-0.081, 0.918, 0.388],
                    [-0.120, 0.964, 0.237],
                    [0.905, 0.404, 0.129],
                    [0.691, -0.365, 0.624],
                    [-0.356, 0.385, 0.852],
                    [-0.363, 0.836, -0.411],
                    [0.920, 0.343, -0.190],
                    [0.754, 0.563, -0.338],
                    [-0.695, 0.628, -0.350],
                    [-0.356, 0.385, 0.852],
                    [-0.081, 0.918, 0.388],
                    [-0.681, 0.192, 0.707],
                    [0.389, -0.867, 0.311],
                    [-0.588, -0.467, 0.661],
                    [0.754, 0.563, -0.338],
                    [0.331, 0.933, 0.138],
                    [0.691, -0.365, 0.624],
                    [-0.530, -0.828, 0.182],
                    [0.389, -0.867, 0.311],
                    [-0.063, 0.070, 0.996],
                    [0.307, -0.946, 0.102],
                    [-0.588, -0.467, 0.661],
                    [-0.588, -0.467, 0.661],
                    [-0.063, 0.070, 0.996],
                    [-0.588, -0.467, 0.661],
                    [-0.432, -0.604, 0.670],
                    [-0.394, -0.023, 0.919],
                    [-0.083, -0.050, 0.995],
                    [-0.867, 0.000, 0.498],
                    [0.060, -0.072, 0.996],
                    [-0.432, -0.604, 0.670],
                    [-0.234, 0.001, 0.972],
                    [-0.081, 0.918, 0.388],
                    [0.202, -0.051, 0.978],
                    [-0.489, 0.792, 0.366],
                    [-0.955, 0.008, 0.297],
                    [0.005, 0.746, 0.665],
                    [0.082, -0.792, 0.606],
                    [0.258, -0.898, 0.356],
                    [0.236, -0.892, 0.385],
                    [-0.786, 0.264, 0.559],
                    [-0.786, 0.264, 0.559],
                    [0.034, -0.765, 0.643],
                    [0.131, -0.854, 0.503],
                    [-0.022, -0.862, 0.506],
                    [0.034, -0.765, 0.643],
                    [0.082, -0.792, 0.606],
                    [0.033, -0.579, 0.814],
                    [0.118, -0.323, 0.939],
                    [0.671, -0.633, 0.386],
                    [-0.046, 0.920, 0.390],
                    [-0.786, 0.264, 0.559],
                    [0.691, -0.365, 0.624],
                    [-0.786, 0.264, 0.559],
                    [0.487, -0.869, -0.083],
                    [-0.022, -0.862, 0.506],
                    [0.451, 0.604, 0.657],
                    [-0.786, 0.264, 0.559],
                    [0.487, -0.869, -0.083],
                    [0.451, 0.604, 0.657],
                    [-0.521, -0.842, 0.142],
                    [0.487, -0.869, -0.083],
                    [-0.786, 0.264, 0.559],
                    [0.487, -0.869, -0.083],
                    [0.671, -0.633, 0.386],
                    [0.451, 0.604, 0.657],
                    [-0.521, -0.842, 0.142],
                    [0.005, 0.746, 0.665],
                    [0.960, 0.073, 0.271],
                    [-0.611, -0.238, -0.755],
                    [0.920, 0.343, -0.190],
                    [0.455, -0.551, 0.700],
                    [0.034, -0.765, 0.643],
                    [0.754, 0.563, -0.338],
                    [0.754, 0.563, -0.338],
                    [0.451, 0.604, 0.657],
                    [0.671, -0.633, 0.386],
                    [0.717, 0.122, 0.687],
                    [0.331, 0.933, 0.138],
                    [-0.046, 0.920, 0.390],
                    [-0.022, -0.862, 0.506],
                    [0.455, -0.551, 0.700],
                    [0.331, 0.933, 0.138],
                    [0.717, 0.122, 0.687],
                    [0.671, -0.633, 0.386],
                    [0.118, -0.323, 0.939],
                    [0.717, 0.122, 0.687],
                    [0.389, -0.867, 0.311],
                    [-0.530, -0.828, 0.182],
                    [0.097, -0.919, 0.383],
                    [0.015, -1.000, 0.008],
                    [-0.246, -0.222, 0.943],
                    [-0.241, -0.970, 0.037],
                    [0.455, -0.551, 0.700],
                    [0.097, -0.919, 0.383],
                    [-0.246, -0.222, 0.943],
                    [0.455, -0.551, 0.700],
                    [0.054, -0.740, 0.671],
                    [0.365, -0.741, 0.564],
                    [0.389, -0.867, 0.311],
                    [0.093, -0.012, 0.996],
                    [0.867, -0.028, 0.498],
                    [-0.046, -0.026, 0.999],
                    [0.920, 0.343, -0.190],
                    [0.034, -0.765, 0.643],
                    [0.754, 0.563, -0.338],
                    [0.054, -0.740, 0.671],
                    [-0.246, -0.222, 0.943],
                    [-0.140, -0.057, 0.988],
                    [0.054, -0.740, 0.671],
                    [0.118, -0.323, 0.939],
                    [-0.212, 0.002, 0.977],
                    [-0.489, 0.792, 0.366],
                    [0.960, 0.073, 0.271],
                    [0.005, 0.746, 0.665],
                    [-0.489, 0.792, 0.366],
                    [0.960, 0.073, 0.271],
                    [-0.298, 0.035, 0.954],
                    [-0.392, 0.025, 0.919],
                    [0.021, 0.956, 0.292],
                    [0.071, 0.959, 0.273],
                    [0.021, 0.956, 0.292],
                    [0.071, 0.959, 0.273],
                    [0.368, -0.028, 0.930],
                    [0.376, -0.034, 0.926],
                    [0.233, 0.031, 0.972],
                    [-0.577, 0.055, 0.815],
                    [-0.926, 0.310, 0.216],
                    [-0.926, 0.310, 0.216],
                    [0.700, -0.014, 0.714],
                    [-0.051, -0.012, 0.999],
                    [0.368, -0.028, 0.930],
                    [0.376, -0.034, 0.926],
                    [-0.568, -0.022, 0.823],
                    [-0.566, -0.023, 0.824],
                    [-0.016, -1.000, 0.011],
                    [-0.009, -0.999, 0.041],
                    [-0.016, -1.000, 0.011],
                    [-0.009, -0.999, 0.041],
                    [0.646, 0.003, 0.763],
                    [0.605, 0.004, 0.796],
                    [-0.298, 0.035, 0.954],
                    [-0.392, 0.025, 0.919],
                    [-0.051, -0.012, 0.999],
                    [-0.467, -0.026, 0.884],
                    [-0.967, 0.072, 0.246],
                    [0.233, 0.031, 0.972],
                    [0.764, -0.000, 0.645],
                    [-0.967, 0.072, 0.246],
                    [0.405, -0.016, 0.914],
                    [0.310, -0.021, 0.950],
                    [-0.026, 0.956, 0.292],
                    [-0.080, 0.957, 0.278],
                    [-0.080, 0.957, 0.278],
                    [-0.026, 0.956, 0.292],
                    [-0.365, -0.011, 0.931],
                    [-0.373, -0.009, 0.928],
                    [0.310, -0.021, 0.950],
                    [-0.236, -0.045, 0.971],
                    [0.587, -0.018, 0.810],
                    [0.587, -0.018, 0.810],
                    [0.941, 0.269, 0.206],
                    [0.941, 0.269, 0.206],
                    [-0.702, -0.045, 0.711],
                    [0.050, 0.014, 0.999],
                    [-0.373, -0.009, 0.928],
                    [-0.365, -0.011, 0.931],
                    [0.571, 0.040, 0.820],
                    [0.500, 0.041, 0.865],
                    [0.136, -0.990, 0.032],
                    [-0.600, -0.034, 0.799],
                    [0.136, -0.990, 0.032],
                    [-0.593, -0.034, 0.804],
                    [0.405, -0.016, 0.914],
                    [0.050, 0.014, 0.999],
                    [0.465, 0.041, 0.884],
                    [0.975, 0.072, 0.210],
                    [-0.772, -0.024, 0.635],
                    [0.975, 0.072, 0.210],
                    [0.321, -0.030, 0.947],
                    [-0.574, -0.022, 0.819],
                    [0.073, 0.994, 0.078],
                    [0.073, 0.994, 0.078],
                    [0.602, 0.005, 0.798],
                    [-0.329, 0.040, 0.943],
                    [-0.329, 0.040, 0.943],
                    [0.315, -0.949, 0.035],
                    [0.315, -0.949, 0.035],
                    [0.321, -0.030, 0.947],
                    [0.283, -0.025, 0.959],
                    [-0.653, -0.019, 0.758],
                    [0.045, 0.994, 0.100],
                    [0.045, 0.994, 0.100],
                    [0.699, 0.002, 0.716],
                    [0.124, 0.034, 0.992],
                    [0.124, 0.034, 0.992],
                    [-0.728, 0.052, 0.684],
                    [-0.075, -0.961, 0.268],
                    [-0.075, -0.961, 0.268],
                    [0.283, -0.025, 0.959],
                    [-0.034, -0.014, 0.999],
                    [-0.547, 0.555, 0.627],
                    [-0.635, -0.019, 0.772],
                    [0.184, -0.980, 0.072],
                    [0.992, -0.125, -0.032],
                    [0.992, -0.125, -0.032],
                    [0.184, -0.980, 0.072],
                    [0.811, -0.002, 0.585],
                    [0.801, -0.002, 0.599],
                    [-0.058, 0.040, 0.998],
                    [-0.323, 0.049, 0.945],
                    [-0.058, 0.040, 0.998],
                    [-0.323, 0.049, 0.945],
                    [-0.869, 0.039, 0.492],
                    [-0.852, 0.041, 0.522],
                    [-0.993, -0.118, -0.001],
                    [0.927, -0.143, 0.347],
                    [-0.993, -0.118, -0.001],
                    [0.927, -0.143, 0.347],
                    [0.554, -0.006, 0.832],
                    [0.420, -0.001, 0.907],
                    [-0.034, -0.014, 0.999],
                    [-0.547, 0.555, 0.627],
                    [0.029, -0.016, 0.999],
                    [-0.679, -0.017, 0.734],
                    [0.006, 0.997, 0.079],
                    [0.006, 0.997, 0.079],
                    [0.759, 0.000, 0.651],
                    [-0.012, 0.041, 0.999],
                    [-0.012, 0.041, 0.999],
                    [-0.831, 0.044, 0.554],
                    [0.815, -0.417, 0.401],
                    [0.815, -0.417, 0.401],
                    [0.029, -0.016, 0.999],
                    [-0.323, -0.005, 0.946],
                    [0.568, 0.040, 0.822],
                    [-0.083, 0.994, 0.078],
                    [-0.083, 0.994, 0.078],
                    [-0.602, -0.033, 0.798],
                    [0.330, 0.001, 0.944],
                    [0.330, 0.001, 0.944],
                    [-0.081, -0.996, 0.033],
                    [-0.081, -0.996, 0.033],
                    [-0.323, -0.005, 0.946],
                    [-0.289, 0.000, 0.957],
                    [0.647, 0.039, 0.761],
                    [-0.057, 0.993, 0.101],
                    [-0.057, 0.993, 0.101],
                    [-0.701, -0.028, 0.713],
                    [-0.131, -0.006, 0.991],
                    [0.726, 0.035, 0.687],
                    [-0.131, -0.006, 0.991],
                    [0.084, -0.960, 0.266],
                    [0.084, -0.960, 0.266],
                    [-0.289, 0.000, 0.957],
                    [-0.547, 0.555, 0.627],
                    [0.011, 0.008, 1.000],
                    [0.693, 0.038, 0.720],
                    [0.707, 0.038, 0.707],
                    [0.080, -0.995, 0.060],
                    [0.768, 0.637, 0.066],
                    [0.768, 0.637, 0.066],
                    [0.080, -0.995, 0.060],
                    [-0.813, 0.095, 0.575],
                    [-0.795, 0.285, 0.536],
                    [0.768, 0.637, 0.066],
                    [-0.813, 0.095, 0.575],
                    [-0.795, 0.285, 0.536],
                    [0.060, 0.002, 0.998],
                    [0.324, 0.018, 0.946],
                    [0.324, 0.018, 0.946],
                    [0.060, 0.002, 0.998],
                    [0.870, 0.027, 0.493],
                    [0.852, 0.028, 0.523],
                    [0.980, -0.199, 0.006],
                    [-0.915, -0.198, 0.351],
                    [-0.915, -0.198, 0.351],
                    [0.980, -0.199, 0.006],
                    [-0.552, -0.035, 0.833],
                    [-0.526, -0.033, 0.850],
                    [0.011, 0.008, 1.000],
                    [-0.547, 0.555, 0.627],
                    [-0.030, 0.008, 0.999],
                    [0.676, 0.039, 0.736],
                    [-0.027, 0.997, 0.077],
                    [-0.027, 0.997, 0.077],
                    [-0.761, -0.025, 0.649],
                    [0.008, 0.003, 1.000],
                    [0.008, 0.003, 1.000],
                    [0.831, 0.029, 0.556],
                    [-0.764, -0.511, 0.394],
                    [-0.764, -0.511, 0.394],
                    [-0.030, 0.008, 0.999],
                    [0.713, 0.182, 0.677],
                    [0.308, 0.183, 0.934],
                    [0.741, 0.629, 0.236],
                    [0.949, -0.267, -0.170],
                    [0.858, -0.366, -0.361],
                    [0.713, 0.182, 0.677],
                    [0.949, -0.267, -0.170],
                    [0.643, 0.241, 0.727],
                    [0.659, -0.693, -0.291],
                    [0.659, -0.693, -0.291],
                    [-0.029, 0.260, 0.965],
                    [0.643, 0.241, 0.727],
                    [0.535, -0.220, 0.815],
                    [0.664, -0.151, 0.732],
                    [-0.428, -0.083, 0.900],
                    [0.308, 0.183, 0.934],
                    [-0.001, -0.088, 0.996],
                    [0.570, 0.034, 0.821],
                    [0.700, 0.029, 0.714],
                    [-0.017, -0.990, 0.138],
                    [-0.978, -0.108, 0.176],
                    [-0.978, -0.108, 0.176],
                    [-0.017, -0.990, 0.138],
                    [-0.478, -0.003, 0.878],
                    [-0.594, 0.039, 0.804],
                    [-0.029, 0.260, 0.965],
                    [0.092, -0.063, 0.994],
                    [-0.330, -0.943, 0.035],
                    [-0.330, -0.943, 0.035],
                    [-0.070, -0.064, 0.995],
                    [-0.304, -0.131, 0.943],
                    [0.916, 0.144, 0.374],
                    [0.916, 0.144, 0.374],
                    [-0.428, -0.083, 0.900],
                    [-0.070, -0.064, 0.995],
                    [-0.428, -0.083, 0.900],
                    [0.664, -0.151, 0.732],
                    [0.535, -0.220, 0.815],
                    [0.934, -0.357, 0.001],
                    [-0.029, 0.260, 0.965],
                    [-0.478, -0.003, 0.878],
                    [0.568, -0.047, 0.822],
                    [0.643, 0.241, 0.727],
                    [-0.029, 0.260, 0.965],
                    [0.568, -0.047, 0.822],
                    [0.643, 0.241, 0.727],
                    [0.580, -0.779, -0.238],
                    [0.858, -0.366, -0.361],
                    [0.858, -0.366, -0.361],
                    [0.580, -0.779, -0.238],
                    [-0.940, -0.111, 0.324],
                    [-0.697, -0.154, 0.700],
                    [0.713, 0.182, 0.677],
                    [-0.001, -0.088, 0.996],
                    [0.308, 0.183, 0.934],
                    [0.063, -0.980, -0.191],
                    [0.063, -0.980, -0.191],
                    [0.092, -0.063, 0.994],
                    [0.934, -0.357, 0.001],
                    [-0.304, -0.131, 0.943],
                    [-0.343, 0.047, 0.938],
                    [0.344, 0.177, 0.922],
                    [-0.741, 0.629, 0.236],
                    [-0.903, -0.356, -0.240],
                    [0.344, 0.177, 0.922],
                    [0.886, -0.126, -0.446],
                    [-0.903, -0.356, -0.240],
                    [0.886, -0.126, -0.446],
                    [-0.393, -0.304, 0.868],
                    [-0.616, -0.714, -0.333],
                    [0.886, -0.126, -0.446],
                    [-0.616, -0.714, -0.333],
                    [-0.903, -0.356, -0.240],
                    [0.157, -0.291, 0.944],
                    [-0.616, -0.714, -0.333],
                    [-0.393, -0.304, 0.868],
                    [-0.664, -0.151, 0.732],
                    [-0.553, -0.152, 0.819],
                    [0.428, -0.076, 0.901],
                    [0.011, -0.166, 0.986],
                    [-0.343, 0.047, 0.938],
                    [-0.527, -0.233, 0.817],
                    [-0.655, -0.242, 0.716],
                    [0.121, -0.983, 0.139],
                    [0.932, -0.305, 0.198],
                    [0.121, -0.983, 0.139],
                    [0.932, -0.305, 0.198],
                    [0.470, -0.077, 0.879],
                    [0.581, -0.072, 0.811],
                    [0.581, -0.072, 0.811],
                    [0.157, -0.291, 0.944],
                    [0.470, -0.077, 0.879],
                    [-0.068, -0.236, 0.970],
                    [0.171, -0.984, 0.051],
                    [0.171, -0.984, 0.051],
                    [0.064, -0.106, 0.992],
                    [0.304, -0.221, 0.927],
                    [-0.893, -0.300, 0.335],
                    [-0.893, -0.300, 0.335],
                    [0.428, -0.076, 0.901],
                    [0.064, -0.106, 0.992],
                    [-0.947, -0.321, -0.001],
                    [-0.578, -0.156, 0.801],
                    [-0.393, -0.304, 0.868],
                    [-0.688, -0.693, -0.215],
                    [0.886, -0.126, -0.446],
                    [-0.688, -0.693, -0.215],
                    [0.344, 0.177, 0.922],
                    [0.693, -0.113, 0.712],
                    [0.886, -0.126, -0.446],
                    [0.011, -0.166, 0.986],
                    [0.344, 0.177, 0.922],
                    [-0.078, -0.978, -0.193],
                    [-0.078, -0.978, -0.193],
                    [-0.068, -0.236, 0.970],
                    [-0.947, -0.321, -0.001],
                    [0.304, -0.221, 0.927],
                    [0.922, -0.121, 0.368],
                    [-0.779, -0.364, 0.510],
                    [0.273, -0.763, 0.586],
                    [-0.690, 0.584, 0.428],
                    [0.092, 0.682, 0.726],
                    [0.922, -0.121, 0.368],
                    [0.959, 0.002, 0.285],
                    [0.092, 0.682, 0.726],
                    [-0.690, 0.584, 0.428],
                    [0.273, -0.763, 0.586],
                    [-0.201, 0.779, 0.595],
                    [-0.916, 0.152, 0.371],
                    [0.447, 0.675, 0.587],
                    [0.986, 0.140, 0.096],
                    [0.447, 0.675, 0.587],
                    [0.700, 0.375, -0.608],
                    [0.986, 0.140, 0.096],
                    [0.700, 0.375, -0.608],
                    [-0.167, -0.082, 0.983],
                    [0.778, -0.398, 0.486],
                    [0.986, 0.140, 0.096],
                    [-0.167, -0.082, 0.983],
                    [-0.167, -0.082, 0.983],
                    [0.273, -0.763, 0.586],
                    [-0.779, -0.364, 0.510],
                    [-0.916, 0.152, 0.371],
                    [-0.779, -0.364, 0.510],
                    [0.092, 0.682, 0.726],
                    [0.447, 0.675, 0.587],
                    [0.092, 0.682, 0.726],
                    [0.959, 0.002, 0.285],
                    [0.273, -0.763, 0.586],
                    [-0.201, 0.779, 0.595],
                    [0.447, 0.675, 0.587],
                    [0.986, 0.140, 0.096],
                    [0.976, -0.181, 0.121],
                    [-0.922, -0.120, 0.369],
                    [-0.344, -0.837, 0.425],
                    [-0.150, 0.793, 0.590],
                    [0.690, 0.584, 0.428],
                    [-0.983, 0.046, 0.180],
                    [-0.922, -0.120, 0.369],
                    [-0.150, 0.793, 0.590],
                    [0.690, 0.584, 0.428],
                    [-0.344, -0.837, 0.425],
                    [0.908, 0.417, 0.048],
                    [0.201, 0.780, 0.593],
                    [-0.444, 0.755, 0.483],
                    [-0.444, 0.755, 0.483],
                    [-0.818, -0.220, 0.531],
                    [-0.992, -0.118, 0.044],
                    [-0.992, -0.118, 0.044],
                    [-0.818, -0.220, 0.531],
                    [0.271, 0.199, 0.942],
                    [-0.512, -0.434, 0.741],
                    [-0.344, -0.837, 0.425],
                    [0.271, 0.199, 0.942],
                    [0.976, -0.181, 0.121],
                    [0.908, 0.417, 0.048],
                    [0.976, -0.181, 0.121],
                    [-0.150, 0.793, 0.590],
                    [-0.150, 0.793, 0.590],
                    [-0.444, 0.755, 0.483],
                    [-0.983, 0.046, 0.180],
                    [-0.444, 0.755, 0.483],
                    [-0.992, -0.118, 0.044],
                    [-0.992, -0.118, 0.044],
                    [0.271, 0.199, 0.942],
                    [-0.344, -0.837, 0.425],
                    [0.201, 0.780, 0.593],
                    [-0.167, -0.918, -0.361],
                    [0.008, -0.193, 0.981],
                    [-0.237, -0.840, 0.489],
                    [-0.237, -0.840, 0.489],
                    [0.008, -0.193, 0.981],
                    [0.020, -0.132, 0.991],
                    [0.014, 0.602, 0.798],
                    [-0.145, -0.955, -0.260],
                    [0.014, 0.602, 0.798],
                    [-0.145, -0.955, -0.260],
                    [0.088, -0.656, 0.750],
                    [0.057, -0.349, 0.935],
                    [0.088, -0.656, 0.750],
                    [0.057, -0.349, 0.935],
                    [-0.104, 0.639, 0.762],
                    [-0.167, -0.918, -0.361],
                    [-0.104, 0.639, 0.762],
                    [-0.167, -0.918, -0.361],
                    [-0.237, -0.840, 0.489],
                    [-0.044, 0.978, -0.204],
                    [0.049, 0.594, 0.803],
                    [0.049, 0.594, 0.803],
                    [-0.104, 0.639, 0.762],
                    [-0.044, 0.978, -0.204],
                    [-0.237, -0.840, 0.489],
                    [0.061, 0.794, 0.605],
                    [0.106, 0.904, 0.415],
                    [-0.061, 0.961, -0.269],
                    [0.106, 0.904, 0.415],
                    [0.986, -0.115, 0.124],
                    [-0.061, 0.961, -0.269],
                    [0.986, -0.115, 0.124],
                    [0.065, -0.662, 0.747],
                    [0.024, 0.031, 0.999],
                    [0.024, 0.028, 0.999],
                    [-0.328, 0.762, 0.559],
                    [-0.097, -0.925, -0.367],
                    [-0.328, 0.762, 0.559],
                    [-0.097, -0.925, -0.367],
                    [0.093, -0.901, 0.425],
                    [0.051, -0.576, 0.816],
                    [0.093, -0.901, 0.425],
                    [0.051, -0.576, 0.816],
                    [0.134, 0.012, 0.991],
                    [0.084, 0.773, 0.629],
                    [0.106, 0.904, 0.415],
                    [0.084, 0.773, 0.629],
                    [0.065, -0.662, 0.747],
                    [-0.061, 0.961, -0.269],
                    [0.026, -0.072, 0.997],
                    [-0.386, 0.727, 0.567],
                    [-0.386, 0.727, 0.567],
                    [-0.328, 0.762, 0.559],
                    [-0.376, -0.866, 0.329],
                    [-0.376, -0.866, 0.329],
                    [-0.376, -0.866, 0.329],
                    [0.061, 0.794, 0.605],
                    [-0.657, 0.689, 0.305],
                    [0.040, 0.776, 0.629],
                    [-0.480, -0.860, 0.172],
                    [-0.255, -0.783, 0.567],
                    [-0.657, 0.689, 0.305],
                    [-0.480, -0.860, 0.172],
                    [-0.314, -0.168, 0.934],
                    [-0.976, -0.033, 0.217],
                    [-0.197, -0.843, 0.500],
                    [-0.314, -0.168, 0.934],
                    [-0.197, -0.843, 0.500],
                    [-0.002, -0.052, 0.999],
                    [0.970, 0.121, 0.212],
                    [0.185, -0.949, -0.254],
                    [0.185, -0.949, -0.254],
                    [0.970, 0.121, 0.212],
                    [-0.070, -0.622, 0.780],
                    [-0.048, -0.349, 0.936],
                    [-0.048, -0.349, 0.936],
                    [-0.070, -0.622, 0.780],
                    [0.996, 0.006, -0.091],
                    [-0.976, -0.033, 0.217],
                    [-0.976, -0.033, 0.217],
                    [0.996, 0.006, -0.091],
                    [-0.197, -0.843, 0.500],
                    [0.118, -0.861, 0.494],
                    [-0.976, -0.026, 0.216],
                    [-0.976, -0.026, 0.216],
                    [0.118, -0.861, 0.494],
                    [-0.255, -0.783, 0.567],
                    [-0.004, 0.031, 1.000],
                    [-0.127, 0.770, 0.625],
                    [-0.127, 0.770, 0.625],
                    [-0.282, -0.858, 0.430],
                    [-0.282, -0.858, 0.430],
                    [-0.058, 0.012, 0.998],
                    [-0.657, 0.689, 0.305],
                    [-0.976, -0.026, 0.216],
                    [-0.255, -0.783, 0.567],
                    [-0.480, -0.860, 0.172],
                    [-0.002, -0.069, 0.998],
                    [0.381, 0.729, 0.568],
                    [-0.127, 0.770, 0.625],
                    [0.381, 0.729, 0.568],
                    [-0.257, -0.836, 0.485],
                    [-0.282, -0.858, 0.430],
                    [-0.127, 0.770, 0.625],
                    [-0.257, -0.836, 0.485],
                    [-0.257, -0.836, 0.485],
                    [0.040, 0.776, 0.629]]

        for e, b in zip(bitangents, expected):
            assert_tuple_values(b, e.as_list())

        """
        pyffi.toaster: INFO:--- fix_addtangentspace - --
        pyffi.toaster: INFO:  ~~~ NiNode[Bip01) ~~~
        pyffi.toaster: INFO:    ~~~ NiNode[Bip01 Pelvis) ~~~
        pyffi.toaster: INFO:      ~~~ NiNode(Bip01 Spine) ~~~
        pyffi.toaster: INFO:        ~~~ NiNode(Bip01 Spine1) ~~~
        pyffi.toaster: INFO:          ~~~ NiNode(Bip01 Spine2) ~~~
        pyffi.toaster: INFO:            ~~~ NiNode(Bip01 Neck) ~~~
        pyffi.toaster: INFO:              ~~~ NiNode(Bip01 Head) ~~~
        pyffi.toaster: INFO:              ~~~ NiNode(Bip01 L Clavicle) ~~~
        pyffi.toaster: INFO:                ~~~ NiNode(Bip01 L UpperArm) ~~~
        pyffi.toaster: INFO:                  ~~~ NiNode(Bip01 L Forearm) ~~~
        pyffi.toaster: INFO:              ~~~ NiNode(Bip01 R Clavicle) ~~~
        pyffi.toaster: INFO:                ~~~ NiNode(Bip01 R UpperArm) ~~~
        pyffi.toaster: INFO:                  ~~~ NiNode(Bip01 R Forearm) ~~~
        pyffi.toaster: INFO:        ~~~ NiNode(Bip01 L Thigh) ~~~
        pyffi.toaster: INFO:          ~~~ NiNode(Bip01 L Calf) ~~~
        pyffi.toaster: INFO:            ~~~ NiNode(Bip01 L Foot) ~~~
        pyffi.toaster: INFO:              ~~~ NiNode(Bip01 L Toe0) ~~~
        pyffi.toaster: INFO:        ~~~ NiNode(Bip01 R Thigh) ~~~
        pyffi.toaster: INFO:          ~~~ NiNode(Bip01 R Calf) ~~~
        pyffi.toaster: INFO:            ~~~ NiNode(Bip01 R Foot) ~~~
        pyffi.toaster: INFO:              ~~~ NiNode(Bip01 R Toe0) ~~~
        pyffi.toaster: INFO:    ~~~ NiTriShape(Body) ~~~
        pyffi.toaster: INFO:      adding tangent space
        """


class TestFixDeltaTangentSpaceNif(BaseNifFileTestCase):
    """Invoke the fix_texturepath spell check through nif toaster"""

    def setUp(self):
        super(TestFixDeltaTangentSpaceNif, self).setUp()
        self.src_name = "test_fix_tangentspace.nif"
        super(TestFixDeltaTangentSpaceNif, self).copyFile()
        super(TestFixDeltaTangentSpaceNif, self).readNifData()

    def test_non_interactive_fix_addtangentspace(self):

        call_niftoaster("--raise", "fix_deltangentspace", "fix_addtangentspace", "--dry-run",  "--noninteractive",
                        "--verbose=1", self.dest_file)
        """
        pyffi.toaster:INFO:=== tests/spells/nif/files/test_fix_tangentspace.nif ===
        pyffi.toaster:INFO:  --- fix_deltangentspace & fix_addtangentspace ---
        pyffi.toaster:INFO:    ~~~ NiNode [Scene Root] ~~~
        pyffi.toaster:INFO:      ~~~ NiTriStrips [Sphere] ~~~
        pyffi.toaster:INFO:        removing tangent space block
        pyffi.toaster:INFO:        adding tangent space
        pyffi.toaster:INFO:  writing to temporary file
        pyffi.toaster:INFO:Finished.
        """

    def test_non_interactive_fix_addtangentspace_series(self):

        call_niftoaster("--raise", "fix_deltangentspace", "fix_addtangentspace", "--series",
                        "--dry-run",  "--noninteractive", "--verbose=1", self.dest_file)

        """
        pyffi.toaster:INFO:=== tests/spells/nif/files/test_fix_tangentspace.nif ===
        pyffi.toaster:INFO:  --- fix_deltangentspace ---
        pyffi.toaster:INFO:    ~~~ NiNode [Scene Root] ~~~
        pyffi.toaster:INFO:      ~~~ NiTriStrips [Sphere] ~~~
        pyffi.toaster:INFO:        removing tangent space block
        pyffi.toaster:INFO:  --- fix_addtangentspace ---
        pyffi.toaster:INFO:    ~~~ NiNode [Scene Root] ~~~
        pyffi.toaster:INFO:      ~~~ NiTriStrips [Sphere] ~~~
        pyffi.toaster:INFO:        adding tangent space
        pyffi.toaster:INFO:  writing to temporary file
        pyffi.toaster:INFO:Finished.
        """